/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT. 
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRENTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 * $LastChangedRevision: 5679 $
 */

/** @file
 * @brief Enhanced ShockBurst Primary Transmitter example
 * @defgroup esb_ptx_example Enhanced ShockBurst Primary Transmitter (PTX) example
 * @{
 * @ingroup nrf_examples
 * 
 * @brief This example sends packets continuously. The contents of P0 is 
 * sent in the first payload byte (byte 0). 
 *
 * The example shows the minimum required setup for transmitting packets to a 
 * primary receiver (PRX) device.  
 * 
 * The following default radio parameters are being used:
 * - RF channel 2
 * - 2 Mbps data rate
 * - TX address 0xE7E7E7E7E7 
 * - 1 byte CRC  
 * 
 * The project @ref esb_prx_example can be used as a counterpart for receiving the data.
 * 
*/

#ifdef MCU_NRF24LE1
#include <Nordic\reg24le1.h>
#endif

#ifdef MCU_NRF24LU1P
#include <Nordic\reg24lu1.h>
#endif

#include <stdint.h>
#include <stdbool.h>
#include "hal_nrf.h"
#include "hal_nrf_hw.h"


// Global variables
uint16_t tx_wait = 0;
bool radio_busy;
bool data_received;

void main()
{
  uint8_t payload[32];
      
  #ifdef MCU_NRF24LU1P
  // Enable radio SPI
  RFCTL = 0x10; 
  #endif

  // Set P1 as output
  P1DIR = 0;
  P1 = 0;

  // Enable the radio clock
  RFCKEN = 1;

  // Enable RF interrupt
  RF = 1; 
   
  // Enable global interrupt
  EA = 1;

    //Enable payload with acknowledgment
  hal_nrf_enable_ack_payload(true);

  //Enable dynamic payload
  hal_nrf_enable_dynamic_payload(true);

  //Enable dynamic acknowledgment on all pipes
  hal_nrf_setup_dynamic_payload(0xFF);
  
  // Power up radio
  hal_nrf_set_power_mode(HAL_NRF_PWR_UP);

  while(1)
  {
    if(tx_wait == 23500){
		// Put P0 contents in payload[0]
    payload[0] = ~P0;
		
    // Write payload to radio TX FIFO
    hal_nrf_write_tx_payload(payload, 3);

    // Toggle radio CE signal to start transmission 
    CE_PULSE();

    // Wait for radio operation to finish
    radio_busy = true;
		tx_wait = 0;
		}
		tx_wait++;
      
    while(radio_busy)
    ;
    if(data_received)
    {
      // Read payload
      while(!hal_nrf_rx_fifo_empty())
      {
      hal_nrf_read_rx_payload(payload);
      }

      // Write received payload[0] to port 1
      P1 = payload[0];
      data_received = false;
    }

  }                                           
} 

// Radio interrupt
void rf_irq() interrupt INTERRUPT_RFIRQ
{
  uint8_t irq_flags;

  // Read and clear IRQ flags from radio
  irq_flags = hal_nrf_get_clear_irq_flags(); 

  switch(irq_flags)
  {
    // Transmission success
    case (1 << HAL_NRF_TX_DS):
      radio_busy = false;
      // Data has been sent
      break;
    // Transmission failed (maximum re-transmits)
    case (1 << HAL_NRF_MAX_RT):
      // When a MAX_RT interrupt occurs the TX payload will not be removed from the TX FIFO. 
      // If the packet is to be discarded this must be done manually by flushing the TX FIFO.
      // Alternatively, CE_PULSE() can be called re-starting transmission of the payload.
      // (Will only be possible after the radio irq flags are cleared) 
      hal_nrf_flush_tx();
      radio_busy = false;
      break;
    case(1 << HAL_NRF_TX_DS | 1 << HAL_NRF_RX_DR):
      radio_busy = false;
      data_received = true;
      break;
  }
}